/*******************************************************************************
 * HMMSegregants - Hidden Markov Model for bulk segregants analysis
 * Copyright 2012 Jorge Duitama
 *
 * This file is part of HMMSegregants.
 *
 *     HMMSegregants is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     HMMSegregants is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with HMMSegregants.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package edu.kuleuven.hmmSegregants;

import java.util.List;

public class BetaParametersEstimator {
	
	public BetaParameter estimateParameters(List<HMMVariant> variants) {
		double sum=0;
		double sum2=0;
		double n = 0;
		for(HMMVariant variant: variants) {
			if(variant.getTotalCount()>1) {
				double x = (double)variant.getAlternativeCount()/(double)variant.getTotalCount();
				//System.out.println("Alternative: "+variant.getAlternativeCount()+" Total: "+variant.getTotalCount()+" x: "+x);
				sum+=x;
				sum2+=(x*x);
				n = n+1;
			}
		}
		double avg = sum/n;
		double variance = (sum2-n*avg*avg)/(n-1);
		//System.err.println("Average: "+avg+" Variance: "+variance);
		double term = avg*(1-avg)/variance - 1;
		double alpha = avg*term;
		double beta = (1-avg)*term;
		return new BetaParameter(alpha, beta);
	}
	
	
}
