package edu.kuleuven.hmmSegregants;

import java.io.IOException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.TreeSet;

public class SegregantSimulatorAutomatic extends SegregantsSimulator {
	
	int causalIndex;
	boolean [][] selectedSeg;
	int [][] counts;
	TreeSet<Integer> snps;
	
	double truePositiveRate = 0.95;
	
	double a;
	double betaP;
	double r;
	
	boolean lowTail = false;
	boolean [][] selectedSeg_lowTail;
	int [][] counts_lowTail;
	
	@Override
	public void simulate() throws IOException {
		// Making segregants
	
		snps = makeSNPs();
		if (snps.size() != nMarkerSites)
			throw new RuntimeException();
		
		//boolean [][] segregants = makeSegregants(snps);
		// Choosing causal sites
		causalIndexes = chooseCausalIndexes();
		causalIndex = getMarkerSite(snps, causalIndexes.keySet().iterator().next());
		//printCausalSites(causalIndexes, snps ,outPrefix+".sites");
		selectedSeg = select(causalIndexes.keySet().iterator().next(), true);
		counts = makeCounts (selectedSeg);
		//printCounts(counts, snps, outPrefix+"_high.counts");
		if (lowTail) {
			selectedSeg_lowTail = select(causalIndexes.keySet().iterator().next(), false);
			counts_lowTail = makeCounts (selectedSeg_lowTail);
		}
	}
	
	private boolean[][] select(int causal, boolean highTail) {
		
		boolean [][] segregants = new boolean[nSelectedSegregants][];
		int i = 0;
		while(i < nSelectedSegregants) {
			boolean[] segregant = makeSegregant(snps);
			double survival_chance = segregant[causal] ? truePositiveRate : 1-truePositiveRate;
			if (!highTail) {
				// change the probability of surviving if selecting for lowTail
				survival_chance = 1 - survival_chance;
			}
			if (random.nextDouble() < survival_chance)  {
				segregants[i++] = segregant;
			}
		}
		return segregants;
	}

	

	List<HMMVariant> buildVariants() {
		List<HMMVariant> variants = new ArrayList<HMMVariant>(nMarkerSites);
		Iterator<Integer> snps_it = snps.iterator();
		for (int i = 0; i < nMarkerSites; i ++) {
			int position = snps_it.next();
			int total = counts[0][i]+counts[1][i];
			int altCount = counts[0][i];
			variants.add(new HMMVariant("ChrSim", position, "0", "1", total, altCount));
		}
		return variants;
	}

	void runHMM(List<HMMVariant> variants) throws IOException {
		
		BetaParametersEstimator estimator = new BetaParametersEstimator();
		BetaParameter paramsB = estimator.estimateParameters(variants);
		BetaParameter paramsP = new BetaParameter(a, betaP);
		
		double alphaPIni = a;
		
		int nIters = 5;
		for(int iter=0;iter < nIters;iter++) {
			List<HMMSegregants> models = HMMSegregants.loadModels(variants,paramsP,paramsB, r, nTotalSegregants);
			List<HMMVariant> variantsB = new ArrayList<HMMVariant>();
			for(HMMSegregants hmm:models) {
				hmm.calculateForward();
				hmm.calculateBackward();
				if (iter == nIters-1) {
					hmm.saveResults(paramsP);
				} else if(a == alphaPIni) {
					variantsB.addAll(hmm.getBackgroundVariants());
				}
			}
			if(variantsB.size()>30) {
				paramsB = estimator.estimateParameters(variantsB);
				//System.err.println("Alpha background: "+paramsB.getAlpha()+" Beta background: "+paramsB.getBeta());
			}
		}
	}

	String getOutput(List<HMMVariant> variants) {
		boolean inPeak = false;
		
		int realPeak_start = 0;
		int realPeak_end = -1;
		
		int peak_start = 0;
		int peak_end = -1;
		
		int falsePeakQty = 0;
		
		boolean causalFound = false;
		
		for(HMMVariant variant:variants) {
			
			if (variant.getPosteriorProbability() > 0.95) {
				if (!inPeak) {
					peak_start = variant.getPosition();
				}
				peak_end = variant.getPosition();
				inPeak = true;	
				
			} else if (inPeak) {
				if (causalFound) {
					
					realPeak_start = peak_start;
					realPeak_end = peak_end;
					
					falsePeakQty --;
					causalFound = false;
				}
				
				falsePeakQty++;
				inPeak = false;
			}
			
			if (inPeak && variant.getPosition() ==  causalIndex) {
				causalFound = true;
				
			}
			
		}
		if (realPeak_end > 0) {
			return "True\t"+(realPeak_end - realPeak_start)+"\t"+falsePeakQty;
		} else {
			return "False\tNA\t"+falsePeakQty;
		}
	}
	
	public static void main(String[] args) throws IOException {
		
		boolean avgQtl = true;
		boolean varyAlpha = false;
		
		PrintStream out = null;
			//outSites.println(""+realMarkerSite+"\t"+causalIndexes.get(i));
		
		if (avgQtl && varyAlpha)
			out = new PrintStream("C:/Users/User/Documents/workspace/HMMSegregants/vary_avgQtl_alpha.txt");
		else if (avgQtl && !varyAlpha)
			out = new PrintStream("C:/Users/User/Documents/workspace/HMMSegregants/vary_avgQtl_rRate.txt");	
		else if (!avgQtl && varyAlpha)
			out = new PrintStream("C:/Users/User/Documents/workspace/HMMSegregants/vary_MarkerQty_alpha.txt");	
		else if (!avgQtl && !varyAlpha)
			out = new PrintStream("C:/Users/User/Documents/workspace/HMMSegregants/vary_MarkerQty_rRate.txt");	
		
		double[] alphas = new double[]{5,10,15,20,25,30,35,40};
		double[] rs = new double[] {
				3.5 * Math.pow(10, -8),
				3.5 * Math.pow(10, -7),
				3.5 * Math.pow(10, -6),
				3.5 * Math.pow(10, -5),
				3.5 * Math.pow(10, -4),
				3.5 * Math.pow(10, -3)};
		
		if (avgQtl) {
			
			System.out.println("Penetrance\tRepeat\t"+(varyAlpha?"alpha":"rRate")+"\tCausalFound\tMainPeakLength\tFP_Peaks");
			out.println("Penetrance\tRepeat\t"+(varyAlpha?"alpha":"rRate")+"\tCausalFound\tMainPeakLength\tFP_Peaks");
			
			//double[] avgQTLEffects = new double[] {0.5, 0.6, 0.65, 0.7, 0.75, 0.8, 0.85, 0.9, 0.95};
			double [] truePositiveRates = new double[] {0.6, 0.7, 0.800, 0.825, 0.850, 0.875, 0.900, 0.925, 0.950, 0.975};
			for (double truePositiveRate: truePositiveRates) {
				
				for (int i = 0; i < 100; i ++) {
					// vary alpha
					
					if (varyAlpha) {
						
						for (double a : alphas) {
							
							SegregantSimulatorAutomatic simulator = new SegregantSimulatorAutomatic();
							
							//simulator.stdevQTLEffect = 0.10;
							simulator.nSelectedSegregants = 30;
							simulator.coverage = 200;
							
							simulator.truePositiveRate = truePositiveRate;
							simulator.nMarkerSites = 2500;
							simulator.simulate();
							
							simulator.a = a;
							simulator.r = 3.5 * Math.pow(10, -6);
							
							List<HMMVariant> variants = simulator.buildVariants();
							simulator.runHMM(variants);
							String output = simulator.getOutput(variants);
							System.out.println(truePositiveRate+"\t"+i+"\t"+a+"\t"+output);
							out.println(truePositiveRate+"\t"+i+"\t"+a+"\t"+output);
							
						}
					} else { // vary R
						
						for (double r : rs) {
							
							SegregantSimulatorAutomatic simulator = new SegregantSimulatorAutomatic();
							
							//simulator.stdevQTLEffect = 0.10;
							simulator.nSelectedSegregants = 30;
							simulator.coverage = 200;
							
							simulator.truePositiveRate = truePositiveRate;
							simulator.nMarkerSites = 2500;
							simulator.simulate();
							
							simulator.a = 30;
							simulator.r = r;
							
							List<HMMVariant> variants = simulator.buildVariants();
							simulator.runHMM(variants);
							String output = simulator.getOutput(variants);
							System.out.println(truePositiveRate+"\t"+i+"\t"+r+"\t"+output);
							out.println(truePositiveRate+"\t"+i+"\t"+r+"\t"+output);
						}
					}
				}
				
			}
		} else {
			
			System.out.println("MarkerQty\tRepeat\t"+(varyAlpha?"alpha":"rRate")+"\tCausalFound\tMainPeakLength\tFP_Peaks");
			out.println("MarkerQty\tRepeat\t"+(varyAlpha?"alpha":"rRate")+"\tCausalFound\tMainPeakLength\tFP_Peaks");
			
			int[] markerQties = new int[] {10, 20, 50, 100, 500, 1000, 2000, 3000, 5000, 7500, 10000};
			for (int markerQty : markerQties) {
				
				for (int i = 0; i < 100; i ++) {
					out.flush();
					
					// vary alpha
					if (varyAlpha) {
						
						for (double a : alphas) {
							
							SegregantSimulatorAutomatic simulator = new SegregantSimulatorAutomatic();
							
							//simulator.stdevQTLEffect = 0.05;
							//simulator.avgQTLEffect = 0.9;
							simulator.nSelectedSegregants = 30;
							simulator.coverage = 200;
							
							simulator.truePositiveRate = 0.95;
							simulator.nMarkerSites = markerQty;
							simulator.simulate();
							
							simulator.a = a;
							simulator.r = 3.5 * Math.pow(10, -6);
							
							List<HMMVariant> variants = simulator.buildVariants();
							simulator.runHMM(variants);
							String output = simulator.getOutput(variants);
							System.out.println(markerQty+"\t"+i+"\t"+a+"\t"+output);
							out.println(markerQty+"\t"+i+"\t"+a+"\t"+output);
							
						}
					} else { // vary R
						
						for (double r : rs) {
							
							SegregantSimulatorAutomatic simulator = new SegregantSimulatorAutomatic();
							
							//simulator.stdevQTLEffect = 0.05;
							//simulator.avgQTLEffect = 0.9;
							simulator.nSelectedSegregants = 30;
							simulator.coverage = 200;
							
							simulator.truePositiveRate = 0.95;
							simulator.nMarkerSites = markerQty;
							simulator.simulate();
							
							simulator.a = 30;
							simulator.r = r;
							
							List<HMMVariant> variants = simulator.buildVariants();
							simulator.runHMM(variants);
							String output = simulator.getOutput(variants);
							
							System.out.println(markerQty+"\t"+i+"\t"+r+"\t"+output);
							out.println(markerQty+"\t"+i+"\t"+r+"\t"+output);
						}
					}
					
				}
				
			}
		}
		
		out.close();
					
	}

	
	
}
